#include <math.h>
#include <stdio.h>

typedef long double r10;

const long MAX_SPHERES_COUNT = 110; //100
const long MAX_SQUARE_EQUATION_SOLUTIONS_COUNT = 2; //2
const r10 eps = 1e-10;

template<class Type>
inline Type Sqr(const Type &a)
{
	return a * a;
}

template<class Type>
inline Type Det2(const Type &a, const Type &b, const Type &c, const Type &d)
{
	return a * d - b * c;
}

inline long SolveQuadraticEquation(r10 a, r10 b, r10 c, r10 solutions[2])
{
	static r10 discriminant;
	discriminant = Sqr(b) - 4 * a * c;
	if(discriminant < 0)return 0;
	solutions[0] = (-b + sqrtl(discriminant)) / (2 * a);
	solutions[1] = (-b - sqrtl(discriminant)) / (2 * a);
	return 2;
}

inline void SolveLinear2
(
	r10 a1, r10 b1, r10 c1,
	r10 a2, r10 b2, r10 c2,
	r10 *x, r10 *y
)
{
	static r10 detX, detY, det;
	det  = Det2( a1,  b1,  a2,  b2);
	detX = Det2(-c1,  b1, -c2,  b2);
	detY = Det2( a1, -c1,  a2, -c2);
	*x = detX / det;
	*y = detY / det;
}

struct tVector3D
{
	r10 x, y, z;

	tVector3D(r10 x, r10 y, r10 z) : x(x), y(y), z(z) {}

	tVector3D(void) : x(0), y(0), z(0) {}

	tVector3D operator* (r10 mult)const
	{
		return tVector3D(x * mult, y * mult, z * mult);
	}

	r10 Length(void)const
	{
		return sqrtl(Sqr(x) + Sqr(y) + Sqr(z));
	}

	tVector3D operator% (const tVector3D& vector)const
	{
		return tVector3D
		(
			Det2(y, z, vector.y, vector.z),
			-Det2(x, z, vector.x, vector.z),
			Det2(x, y, vector.x, vector.y)
		);
	}

	void Normalize(void)
	{
		r10 multiplier = Length();
		x /= multiplier;
		y /= multiplier;
		z /= multiplier;
	}

	r10 operator* (const tVector3D& vector)const
	{
		return 	x * vector.x +
				y * vector.y +
				z * vector.z;
	}

	void Read(void)
	{
		scanf("%Lf %Lf %Lf", &x, &y, &z);
	}

	tVector3D operator- (const tVector3D& vector)const
	{
		return tVector3D(x - vector.x, y - vector.y, z - vector.z);
	}

	tVector3D operator+ (const tVector3D& vector)const
	{
		return tVector3D(vector.x + x, vector.y + y, vector.z + z);
	}

	r10 DistanceTo(const tVector3D& vector)const
	{
		return (vector - *this).Length();
	}
};

struct tLine3D;

struct tPlane
{
	tVector3D normal;
	r10 D;

	tPlane(void) : D(0){}

	tPlane(r10 A, r10 B, r10 C, r10 D) : normal(A, B, C), D(D) {}

	inline tPlane(const tLine3D& line, const tVector3D& point);
	inline tVector3D IntersectionWithLine(const tLine3D& line)const;

	int ParallelWith(const tPlane& plane)const
	{
		if((normal % plane.normal).Length() < eps)return 1;
		else return 0;
	}

	void Normalize(void)
	{
		static r10 vectorLen;
		vectorLen = normal.Length();
		normal.Normalize();
		D /= vectorLen;
	}
};

struct tLine3D
{
	tVector3D start;
	tVector3D direction;

	tLine3D(void) {}

	tLine3D(const tVector3D& point1, const tVector3D& point2)
	{
		start = point1;
		direction = point2 - point1;
	}

	tLine3D(const tPlane &plane1, const tPlane &plane2)
	{
		if
		(
			fabs
			(
				Det2
				(
					plane1.normal.x, plane2.normal.x,
					plane1.normal.y, plane2.normal.y
				)
			) > eps
		)
		{
			SolveLinear2
			(
				plane1.normal.x, plane1.normal.y, plane1.D,
				plane2.normal.x, plane2.normal.y, plane2.D,
				&start.x, &start.y
			);
			start.z = 0;
		}
		else
		{
			start.x = 0;
			SolveLinear2
			(
				plane1.normal.y, plane1.normal.z, plane1.D,
				plane2.normal.y, plane2.normal.z, plane2.D,
				&start.y, &start.z
			);
		}
		direction = (plane1.normal % plane2.normal);
		direction.Normalize();
	}
};

inline tPlane::tPlane(const tLine3D& line, const tVector3D& point)
{
	normal = line.direction;
	D = - (normal * point);
}

inline tVector3D tPlane::IntersectionWithLine(const tLine3D& line)const
{
	static r10 parameter;
	parameter = -
		(normal * line.start + D) /
		(normal * line.direction);
	return line.start + line.direction * parameter;
}

struct tSphere
{
	tVector3D center;
	r10 radius;

	void Read()
	{
		center.Read();
		scanf("%Lf", &radius);
		radius += eps;
	}

	int Intersects(const tLine3D& line)const
	{
		static tPlane plane;
		plane = tPlane(line, center);

		static tVector3D perpendicularEnd;
		perpendicularEnd = plane.IntersectionWithLine(line);

		static r10 distance;
		distance = perpendicularEnd.DistanceTo(center);

		return distance < radius + eps;
	}

};

struct tCircle3D
{
	tVector3D center;
	r10 radius;
	tPlane plane;

	void Intersect(const tCircle3D& circle, tVector3D* points, long* cPoints)const
	{
		if(plane.ParallelWith(circle.plane))return;
		static tLine3D line;
		line = tLine3D(plane, circle.plane);

		static r10 a, b, c;
		a = Sqr(line.direction.Length());
		b = 2 * ((line.start - center) * line.direction);
		c = Sqr((line.start - center).Length()) - Sqr(radius);

		static r10 solutions[2];
		static long cSolutions, iSolution;

		cSolutions = SolveQuadraticEquation(a, b, c, solutions);
		for(iSolution = 0; iSolution < cSolutions; ++iSolution)
			points[(*cPoints)++] = line.start +
				line.direction * solutions[iSolution];
	}
};

struct tCentralProjectionSphere
{
	tCircle3D Project(const tSphere& sphere)
	{
		static tCircle3D result;

		static r10 tangentLength;
		tangentLength = sqrtl(Sqr(sphere.center.Length()) - Sqr(sphere.radius));
		result.center = sphere.center * (tangentLength / Sqr(sphere.center.Length()));

		result.plane.normal = sphere.center;
		result.plane.D = - (result.plane.normal * result.center);
		result.plane.Normalize();

		result.radius = sphere.radius / sphere.center.Length();

		return result;
	}
};


long CheckIntersections(const tSphere *spheres, long cSpheres, const tVector3D& point)
{
	static long cIntersections, iSphere;
	cIntersections = 0;
	for(cIntersections = 0, iSphere = 0; iSphere < cSpheres; ++iSphere)
		if
		(
			spheres[iSphere].Intersects
				(tLine3D(tVector3D(), point))
		)
		++cIntersections;
	return cIntersections;
}

int main(void)
{
	static tSphere spheres[MAX_SPHERES_COUNT];
	static tCentralProjectionSphere centralProjectionSphere;
	static tCircle3D circles[MAX_SPHERES_COUNT];
	static tVector3D points[MAX_SQUARE_EQUATION_SOLUTIONS_COUNT];

	freopen("ghosts.in", "r", stdin);
	freopen("ghosts.out", "w", stdout);

	long iSphere, cSpheres;
	scanf("%ld", &cSpheres);
	for(iSphere = 0; iSphere < cSpheres; ++iSphere)
		spheres[iSphere].Read();

	for(iSphere = 0; iSphere < cSpheres; ++iSphere)
		circles[iSphere] = centralProjectionSphere.Project(spheres[iSphere]);
	long cCircles = cSpheres;

	tVector3D maxIntersectionsPoint;
	long cIntersectionsCount;
	long cMaxIntersections = 0;
	long iPoint, cPoints;
	long iCircle1, iCircle2;
	for(iCircle1 = 0; iCircle1 < cCircles; ++iCircle1)
		for(iCircle2 = iCircle1 + 1; iCircle2 < cCircles; ++iCircle2)
		{
			cPoints = 0;
			circles[iCircle1].Intersect(circles[iCircle2], points, &cPoints);

			for(iPoint = 0; iPoint < cPoints; ++iPoint)
			{
				cIntersectionsCount = CheckIntersections(spheres, cSpheres, points[iPoint]);
				if(cIntersectionsCount > cMaxIntersections)
				{
					cMaxIntersections = cIntersectionsCount;
					maxIntersectionsPoint = points[iPoint];
				}
			}
		}

	for(iSphere = 0; iSphere < cSpheres; ++iSphere)
	{
		cIntersectionsCount =
			CheckIntersections(spheres, cSpheres, spheres[iSphere].center);
		if(cIntersectionsCount > cMaxIntersections)
		{
			cMaxIntersections = cIntersectionsCount;
			maxIntersectionsPoint = spheres[iSphere].center;
		}
	}

	printf("%ld\n", cMaxIntersections);
	int firstNumber = 1;
	for(iSphere = 0; iSphere < cSpheres; ++iSphere)
		if
		(
			spheres[iSphere].Intersects
				(tLine3D(tVector3D(), maxIntersectionsPoint))
		)
		{
			if(firstNumber)firstNumber = 0;
			else putchar('\n');
			printf("%ld", iSphere + 1);
		}

	fclose(stdout);
	fclose(stdin);

	return 0;
}